@@ -33,9 +33,6 @@
  */
 #define CLIENT_CLEAN_UP_TIME  300
 
-/* How often we check for stale client cache (in seconds) */
-#define CLIENT_CLEAN_UP_CHECK 60
-
 struct TrackerDBusRequestHandler {
 	TrackerDBusRequestFunc new;
 	TrackerDBusRequestFunc done;
@@ -47,6 +44,7 @@ typedef struct {
 	gchar *binary;
 	gulong pid;
 	GTimeVal last_time;
+	guint clean_up_id;
 } ClientData;
 
 typedef struct {
@@ -56,6 +54,7 @@ typedef struct {
 	DBusPendingCall *call;
 	TrackerDBusSendAndSpliceCallback callback;
 	gpointer user_data;
+	gboolean expect_variable_names;
 } SendAndSpliceData;
 
 static GSList *hooks;
@@ -65,10 +64,9 @@ static gboolean client_lookup_enabled;
 static DBusGConnection *freedesktop_connection;
 static DBusGProxy *freedesktop_proxy;
 static GHashTable *clients;
-static guint clients_clean_up_id;
 
 static void     client_data_free    (gpointer data);
-static gboolean clients_clean_up_cb (gpointer data);
+static gboolean client_clean_up_cb (gpointer data);
 
 static void
 request_handler_call_for_new (guint request_id)
@@ -144,8 +142,6 @@ clients_init (void)
 	                                 g_str_equal,
 	                                 NULL,
 	                                 client_data_free);
-	clients_clean_up_id =
-		g_timeout_add_seconds (CLIENT_CLEAN_UP_CHECK, clients_clean_up_cb, NULL);
 
 	return TRUE;
 }
@@ -163,11 +159,6 @@ clients_shutdown (void)
 		freedesktop_connection = NULL;
 	}
 
-	if (clients_clean_up_id != 0) {
-		g_source_remove (clients_clean_up_id);
-		clients_clean_up_id = 0;
-	}
-
 	if (clients) {
 		g_hash_table_unref (clients);
 		clients = NULL;
@@ -185,6 +176,8 @@ client_data_free (gpointer data)
 		return;
 	}
 
+	g_source_remove (cd->clean_up_id);
+
 	g_free (cd->sender);
 	g_free (cd->binary);
 
@@ -248,44 +241,22 @@ client_data_new (gchar *sender)
 }
 
 static gboolean
-clients_clean_up_cb (gpointer data)
+client_clean_up_cb (gpointer data)
 {
-	GTimeVal now;
-	GHashTableIter iter;
-	gpointer key, value;
+	ClientData *cd;
 
-	g_get_current_time (&now);
+	cd = data;
 
-	g_hash_table_iter_init (&iter, clients);
-	while (g_hash_table_iter_next (&iter, &key, &value)) {
-		ClientData *cd;
-		glong diff;
-
-		cd = value;
-
-		diff = now.tv_sec - cd->last_time.tv_sec;
-
-		/* 5 Minutes */
-		if (diff >= CLIENT_CLEAN_UP_TIME) {
-			g_debug ("Removing D-Bus client data for '%s' with id:'%s'",
-			         cd->binary, cd->sender);
-			g_hash_table_iter_remove (&iter);
-		}
-	}
+	g_debug ("Removing D-Bus client data for '%s' with id:'%s'",
+	         cd->binary, cd->sender);
+	g_hash_table_remove (clients, cd->sender);
 
 	if (g_hash_table_size (clients) < 1) {
-		/* This must be before the clients_shutdown which will
-		 * attempt to clean up the the timeout too.
-		 */
-		clients_clean_up_id = 0;
-
-		/* Clean everything else up. */
+		/* Clean everything up. */
 		clients_shutdown ();
-
-		return FALSE;
 	}
 
-	return TRUE;
+	return FALSE;
 }
 
 static ClientData *
@@ -320,8 +291,11 @@ client_get_for_context (DBusGMethodInvocation *context)
 		g_hash_table_insert (clients, sender, cd);
 	} else {
 		g_free (sender);
+		g_source_remove (cd->clean_up_id);
 	}
 
+	cd->clean_up_id = g_timeout_add_seconds (CLIENT_CLEAN_UP_TIME, client_clean_up_cb, cd);
+
 	g_get_current_time (&cd->last_time);
 
 	return cd;
@@ -705,6 +679,31 @@ tracker_dbus_enable_client_lookup (gboolean enabled)
 	client_lookup_enabled = enabled;
 }
 
+static GStrv
+dbus_send_and_splice_get_variable_names (DBusMessage *message,
+                                         gboolean     copy_strings)
+{
+	GPtrArray *found;
+	DBusMessageIter iter, arr;
+
+	dbus_message_iter_init (message, &iter);
+	dbus_message_iter_recurse (&iter, &arr);
+
+	found = g_ptr_array_new ();
+
+	while (dbus_message_iter_get_arg_type (&arr) != DBUS_TYPE_INVALID) {
+		gchar *str;
+
+		dbus_message_iter_get_basic (&arr, &str);
+		g_ptr_array_add (found, copy_strings ? g_strdup (str) : str);
+		dbus_message_iter_next (&arr);
+	}
+
+	g_ptr_array_add (found, NULL);
+
+	return (GStrv) g_ptr_array_free (found, FALSE);
+}
+
 /*
  * /!\ BIG FAT WARNING /!\
  * The message must be destroyed for this function to succeed, so pass a
@@ -718,6 +717,7 @@ tracker_dbus_send_and_splice (DBusConnection  *connection,
                               GCancellable    *cancellable,
                               void           **dest_buffer,
                               gssize          *dest_buffer_size,
+                              GStrv           *variable_names,
                               GError         **error)
 {
 	DBusPendingCall *call;
@@ -784,6 +784,10 @@ tracker_dbus_send_and_splice (DBusConnection  *connection,
 				*dest_buffer_size = g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (output_stream));
 			}
 
+			if (variable_names) {
+				*variable_names = dbus_send_and_splice_get_variable_names (reply, TRUE);
+			}
+
 			ret_value = TRUE;
 		}
 	} else {
@@ -814,6 +818,7 @@ static SendAndSpliceData *
 send_and_splice_data_new (GInputStream                     *unix_input_stream,
                           GInputStream                     *buffered_input_stream,
                           GOutputStream                    *output_stream,
+                          gboolean                          expect_variable_names,
                           DBusPendingCall                  *call,
                           TrackerDBusSendAndSpliceCallback  callback,
                           gpointer                          user_data)
@@ -827,6 +832,7 @@ send_and_splice_data_new (GInputStream                     *unix_input_stream,
 	data->call = call;
 	data->callback = callback;
 	data->user_data = user_data;
+	data->expect_variable_names = expect_variable_names;
 
 	return data;
 }
@@ -870,25 +876,36 @@ send_and_splice_async_callback (GObject      *source,
 			dbus_set_g_error (&error, &dbus_error);
 			dbus_error_free (&dbus_error);
 
-			(* data->callback) (NULL, -1, error, data->user_data);
+			(* data->callback) (NULL, -1, NULL, error, data->user_data);
 
 			/* Note: GError should be freed by callback. We do this to be aligned
 			 * with the behavior of dbus-glib, where if an error happens, the
 			 * GError passed to the callback is supposed to be disposed by the
 			 * callback itself. */
 		} else {
+			GStrv v_names = NULL;
+
+			if (data->expect_variable_names) {
+				v_names = dbus_send_and_splice_get_variable_names (reply, FALSE);
+			}
+
 			dbus_pending_call_cancel (data->call);
+
 			(* data->callback) (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (data->output_stream)),
 			                    g_memory_output_stream_get_data_size (G_MEMORY_OUTPUT_STREAM (data->output_stream)),
+			                    v_names,
 			                    NULL,
 			                    data->user_data);
+
+			/* Don't use g_strfreev here, see above */
+			g_free (v_names);
 		}
 	} else {
 		/* If any error happened, we're not passing any received data, so we
 		 * need to free it */
 		g_free (g_memory_output_stream_get_data (G_MEMORY_OUTPUT_STREAM (data->output_stream)));
 
-		(* data->callback) (NULL, -1, error, data->user_data);
+		(* data->callback) (NULL, -1, NULL, error, data->user_data);
 
 		/* Note: GError should be freed by callback. We do this to be aligned
 		 * with the behavior of dbus-glib, where if an error happens, the
@@ -907,6 +924,7 @@ gboolean
 tracker_dbus_send_and_splice_async (DBusConnection                   *connection,
                                     DBusMessage                      *message,
                                     int                               fd,
+                                    gboolean                          expect_variable_names,
                                     GCancellable                     *cancellable,
                                     TrackerDBusSendAndSpliceCallback  callback,
                                     gpointer                          user_data)
@@ -941,6 +959,7 @@ tracker_dbus_send_and_splice_async (DBusConnection                   *connection
 	data = send_and_splice_data_new (unix_input_stream,
 	                                 buffered_input_stream,
 	                                 output_stream,
+	                                 expect_variable_names,
 	                                 call,
 	                                 callback,
 	                                 user_data);
