@@ -24,6 +24,10 @@
 
 #include "tracker-miner-applications.h"
 
+#ifdef HAVE_MEEGOTOUCH
+#include "tracker-miner-applications-meego.h"
+#endif
+
 #define GROUP_DESKTOP_ENTRY          "Desktop Entry"
 
 #define APPLICATION_DATASOURCE_URN   "urn:nepomuk:datasource:84f20000-1241-11de-8c30-0800200c9a66"
@@ -31,19 +35,23 @@
 #define SOFTWARE_CATEGORY_URN_PREFIX "urn:software-category:"
 #define THEME_ICON_URN_PREFIX        "urn:theme-icon:"
 
-static void     miner_applications_finalize          (GObject              *object);
-static void     miner_applications_constructed       (GObject              *object);
+static void     miner_applications_finalize                (GObject              *object);
+static void     miner_applications_constructed             (GObject              *object);
 
-static gboolean miner_applications_check_file        (TrackerMinerFS       *fs,
-                                                      GFile                *file);
-static gboolean miner_applications_check_directory   (TrackerMinerFS       *fs,
-                                                      GFile                *file);
-static gboolean miner_applications_process_file      (TrackerMinerFS       *fs,
-                                                      GFile                *file,
-                                                      TrackerSparqlBuilder *sparql,
-                                                      GCancellable         *cancellable);
-static gboolean miner_applications_monitor_directory (TrackerMinerFS       *fs,
-                                                      GFile                *file);
+static gboolean miner_applications_check_file              (TrackerMinerFS       *fs,
+                                                            GFile                *file);
+static gboolean miner_applications_check_directory         (TrackerMinerFS       *fs,
+                                                            GFile                *file);
+static gboolean miner_applications_process_file            (TrackerMinerFS       *fs,
+                                                            GFile                *file,
+                                                            TrackerSparqlBuilder *sparql,
+                                                            GCancellable         *cancellable);
+static gboolean miner_applications_process_file_attributes (TrackerMinerFS       *fs,
+                                                            GFile                *file,
+                                                            TrackerSparqlBuilder *sparql,
+                                                            GCancellable         *cancellable);
+static gboolean miner_applications_monitor_directory       (TrackerMinerFS       *fs,
+                                                            GFile                *file);
 
 static GQuark miner_applications_error_quark = 0;
 
@@ -73,6 +81,7 @@ tracker_miner_applications_class_init (TrackerMinerApplicationsClass *klass)
 	miner_fs_class->check_directory = miner_applications_check_directory;
 	miner_fs_class->monitor_directory = miner_applications_monitor_directory;
 	miner_fs_class->process_file = miner_applications_process_file;
+	miner_fs_class->process_file_attributes = miner_applications_process_file_attributes;
 
 	miner_applications_error_quark = g_quark_from_static_string ("TrackerMinerApplications");
 }
@@ -103,6 +112,12 @@ miner_applications_constructed (GObject *object)
 	tracker_miner_fs_directory_add (TRACKER_MINER_FS (object), file, TRUE);
 	g_object_unref (file);
 
+#ifdef HAVE_MEEGOTOUCH
+	file = g_file_new_for_path ("/usr/lib/duicontrolpanel/");
+	tracker_miner_fs_directory_add (TRACKER_MINER_FS (object), file, TRUE);
+	g_object_unref (file);
+#endif /* HAVE_MEEGOTOUCH */
+
 	/* FIXME: Check XDG_DATA_DIRS and also process applications in there */
 }
 
@@ -112,7 +127,7 @@ insert_data_from_desktop_file (TrackerSparqlBuilder *sparql,
                                const gchar          *metadata_key,
                                GKeyFile             *desktop_file,
                                const gchar          *key,
-                               gboolean                      use_locale)
+                               gboolean              use_locale)
 {
 	gchar *str;
 
@@ -266,11 +281,19 @@ process_desktop_file (ProcessApplicationData  *data,
 {
 	TrackerSparqlBuilder *sparql;
 	GKeyFile *key_file;
-	gchar *path, *type, *filename, *name, *uri = NULL;
+	gchar *name = NULL;
+	gchar *path;
+	gchar *type;
+	gchar *filename;
+	gchar *uri = NULL;
 	GStrv cats;
 	gsize cats_len;
 	gboolean is_software = TRUE;
 	const gchar *parent_urn;
+#ifdef HAVE_MEEGOTOUCH
+	gchar *logical_id = NULL;
+	gchar *translation_catalog = NULL;
+#endif /* HAVE_MEEGOTOUCH */
 
 	sparql = data->sparql;
 	key_file = data->key_file;
@@ -286,8 +309,23 @@ process_desktop_file (ProcessApplicationData  *data,
 
 	/* NOTE: We sanitize categories later on when iterating them */
 
-	/* Try to get the name with locale, then without if that fails */
-	name = g_key_file_get_locale_string (key_file, GROUP_DESKTOP_ENTRY, "Name", NULL, NULL);
+#ifdef HAVE_MEEGOTOUCH
+	/* If defined, start with the logical strings */
+	logical_id = g_key_file_get_string (key_file, GROUP_DESKTOP_ENTRY, "X-MeeGo-Logical-Id", NULL);
+	translation_catalog = g_key_file_get_string (key_file, GROUP_DESKTOP_ENTRY, "X-MeeGo-Translation-Catalog", NULL);
+
+	if (logical_id && translation_catalog) {
+		name = tracker_miner_applications_meego_translate (translation_catalog, logical_id);
+	}
+
+	g_free (logical_id);
+	g_free (translation_catalog);
+#endif /* HAVE_MEEGOTOUCH */
+
+	if (!name) {
+		/* Try to get the name with locale, then without if that fails */
+		name = g_key_file_get_locale_string (key_file, GROUP_DESKTOP_ENTRY, "Name", NULL, NULL);
+	}
 
 	if (!name) {
 		name = g_key_file_get_string (key_file, GROUP_DESKTOP_ENTRY, "Name", NULL);
@@ -330,7 +368,6 @@ process_desktop_file (ProcessApplicationData  *data,
 		}
 
 		is_software = FALSE;
-
 	} else if (name && g_ascii_strcasecmp (type, "Application") == 0) {
 		uri = g_file_get_uri (data->file);
 		tracker_sparql_builder_insert_silent_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
@@ -347,6 +384,25 @@ process_desktop_file (ProcessApplicationData  *data,
 
 		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
 		tracker_sparql_builder_object_iri (sparql, APPLICATION_DATASOURCE_URN);
+#ifdef HAVE_MEEGOTOUCH
+	} else if (name && g_ascii_strcasecmp (type, "ControlPanelApplet") == 0) {
+		/* Special case control panel applets */
+		/* The URI of the InformationElement should be a UUID URN */
+		uri = g_file_get_uri (data->file);
+		tracker_sparql_builder_insert_silent_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
+
+		tracker_sparql_builder_subject_iri (sparql, APPLET_DATASOURCE_URN);
+		tracker_sparql_builder_predicate (sparql, "a");
+		tracker_sparql_builder_object (sparql, "nie:DataSource");
+
+		/* TODO This is atm specific for Maemo */
+		tracker_sparql_builder_subject_iri (sparql, uri);
+
+		tracker_sparql_builder_predicate (sparql, "a");
+		tracker_sparql_builder_object (sparql, "maemo:ControlPanelApplet");
+
+		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
+		tracker_sparql_builder_object_iri (sparql, APPLET_DATASOURCE_URN);
 
 		/* This matches SomeApplet as Type= */
 	} else if (name && g_str_has_suffix (type, "Applet")) {
@@ -366,6 +422,25 @@ process_desktop_file (ProcessApplicationData  *data,
 
 		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
 		tracker_sparql_builder_object_iri (sparql, APPLET_DATASOURCE_URN);
+
+	} else if (name && g_ascii_strcasecmp (type, "DUIApplication") == 0) {
+
+		uri = g_file_get_uri (data->file);
+		tracker_sparql_builder_insert_silent_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
+
+		tracker_sparql_builder_subject_iri (sparql, APPLICATION_DATASOURCE_URN);
+		tracker_sparql_builder_predicate (sparql, "a");
+		tracker_sparql_builder_object (sparql, "nie:DataSource");
+
+		tracker_sparql_builder_subject_iri (sparql, uri);
+
+		tracker_sparql_builder_predicate (sparql, "a");
+		tracker_sparql_builder_object (sparql, "nfo:SoftwareApplication");
+		tracker_sparql_builder_object (sparql, "nie:DataObject");
+
+		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
+		tracker_sparql_builder_object_iri (sparql, APPLICATION_DATASOURCE_URN);
+#endif /* HAVE_MEEGOTOUCH */
 	} else {
 		/* Invalid type, all valid types are already listed above */
 		uri = g_file_get_uri (data->file);
@@ -381,9 +456,25 @@ process_desktop_file (ProcessApplicationData  *data,
 		tracker_sparql_builder_object (sparql, "nfo:SoftwareApplication");
 		tracker_sparql_builder_object (sparql, "nie:DataObject");
 
-		g_debug ("Invalid desktop file:'%s'", uri);
-		g_debug ("  Type '%s' is not part of the desktop file specification (expected 'Application', 'Link' or 'Directory')", type);
-		g_debug ("  Defaulting to 'Application'");
+		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
+		tracker_sparql_builder_object_iri (sparql, APPLICATION_DATASOURCE_URN);
+
+		if (name) {
+			/* If we got a name, then the issue comes from the type.
+			 * As we're defaulting to Application here, we just g_debug() the problem. */
+			g_debug ("Invalid desktop file: '%s'", uri);
+			g_debug ("  Type '%s' is not part of the desktop file specification (expected 'Application', 'Link' or 'Directory')", type);
+			g_debug ("  Defaulting to 'Application'");
+		} else {
+			/* If we didn't get a name, the problem is more severe as we don't default it
+			 * to anything, so we g_warning() it.  */
+			g_warning ("Invalid desktop file: '%s'", uri);
+#ifdef HAVE_MEEGOTOUCH
+			g_warning ("  Couldn't get name, missing or wrong key (X-MeeGo-Logical-Id, X-MeeGo-Translation-Catalog or Name)");
+#else
+			g_warning ("  Couldn't get name, missing key (Name)");
+#endif
+		}
 	}
 
 	if (sparql && uri) {
@@ -402,8 +493,14 @@ process_desktop_file (ProcessApplicationData  *data,
 		   tracker_sparql_builder_predicate (sparql, "tracker:available");
 		   tracker_sparql_builder_object_boolean (sparql, TRUE); */
 
-		tracker_sparql_builder_predicate (sparql, "nie:title");
-		tracker_sparql_builder_object_string (sparql, name);
+		/* We should always always have a proper name if the desktop file is correct
+		 * w.r.t to the Meego or Freedesktop specs, but sometimes this is not true,
+		 * so instead of passing wrong stuff to the SPARQL builder, we avoid it.
+		 * If we don't have a proper name, we already warned it before. */
+		if (name) {
+			tracker_sparql_builder_predicate (sparql, "nie:title");
+			tracker_sparql_builder_object_string (sparql, name);
+		}
 
 		if (is_software) {
 			gchar *icon;
@@ -476,10 +573,6 @@ process_desktop_file (ProcessApplicationData  *data,
 			}
 		}
 
-		tracker_sparql_builder_subject_iri (sparql, uri);
-		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
-		tracker_sparql_builder_object_iri (sparql, APPLICATION_DATASOURCE_URN);
-
 		filename = g_filename_display_basename (path);
 		tracker_sparql_builder_predicate (sparql, "nfo:fileName");
 		tracker_sparql_builder_object_string (sparql, filename);
@@ -615,6 +708,22 @@ miner_applications_process_file (TrackerMinerFS       *fs,
 	return TRUE;
 }
 
+static gboolean
+miner_applications_process_file_attributes (TrackerMinerFS       *fs,
+                                            GFile                *file,
+                                            TrackerSparqlBuilder *sparql,
+                                            GCancellable         *cancellable)
+{
+	gchar *uri;
+
+	/* We don't care about file attribute changes here */
+	uri = g_file_get_uri (file);
+	g_debug ("Ignoring file attribute changes in '%s'", uri);
+	g_free (uri);
+
+	return FALSE;
+}
+
 TrackerMiner *
 tracker_miner_applications_new (void)
 {
